-- Title       : EFX_BEAM_TEST.MS
-- Category    : Geometry
-- Environment : 3ds max 4
-- Description : EffectWare Beams Plug-in Testing
-- Reference   :

-- Copyright (c) 2001, EffectWare, Moon-Sun Choi
-- http://www.effectware.com
-- aeroplus@effectware.com

-- general functions ----------------------------------------------------------

-- draw H-Beam using 2 point
-- sp : start point
-- ep : end point
-- h : height
-- w : width
-- t1 : thickness1
-- t2 : thickness2
-- r1 : radius1
-- r2 : radius2
-- uw : unit weight

fn draw_h_beam_2p sp ep h w t1 t2 r1 r2 uw =
(
	local len
	local dir
	local mat
	local beam

	len = length (ep - sp)
	dir = normalize (ep - sp)
	mat = matrixFromNormal dir

	-- change this
	beam = HBeam pos:sp height:h width:w thickness1:t1 thickness2:t2 radius1:r1 radius2:r2 unit_weight:uw
	-- rotate beam
	rotate beam (mat as eulerAngles)
	-- extrude beam
	addModifier beam (Extrude amount:len)

	beam
)

-- draw H-Beam using 1 point, angle, and length
-- sp : start point
-- ang : rotation angle
-- len : length
-- h : height
-- w : width
-- t1 : thickness1
-- t2 : thickness2
-- r1 : radius1
-- r2 : radius2
-- uw : unit weight

fn draw_h_beam sp ang len h w t1 t2 r1 r2 uw =
(
	local mat
	local beam

	-- create beam
	beam = HBeam height:h width:w thickness1:t1 thickness2:t2 radius1:r1 radius2:r2 unit_weight:uw
	-- rotate beam
	rotate beam (eulerAngles ang.x ang.y ang.z)
	-- move beam
	move beam sp
	-- extrude beam
	addModifier beam (Extrude amount:len)

	beam
)

-- array Beam
-- beam : Beam Object
-- nx : x counts
-- ny : y counts
-- nz : z counts
-- dx : x spacing
-- dy : y spacing
-- dz : z spacing
-- status : status label

fn draw_beam_array beam nx ny nz dx dy dz status =
(
	local sp
	local x, y, z
	local cx, cy, cz
	local bclone
	local col

	sp = beam.pos

	cz = sp.z
	z = 0
	while z < nz do
	(
		cy = sp.y
		y = 0
		while y < ny do
		(
			cx = sp.x
			x = 0
			while x < nx do
			(
				status.text = "Processing: " + ((x + 1) as string) + "," + ((y + 1) as string) + "," + ((z + 1) as string)
				if col == undefined then
				(
					col = beam.wireColor
				)
				else
				(
					bclone = instance beam
					bclone.pos = [cx, cy, cz]
					bclone.wireColor = col
				)
				cx += dx
				x += 1
			)
			cy += dy
			y += 1
		)
		cz += dz
		z += 1
	)
)

-- draw steel structure
-- nx : x counts
-- ny : y counts
-- nz : z counts
-- dx : x spacing
-- dy : y spacing
-- dz : z spacing
-- status : status label

fn draw_steel_structure nx ny nz dx dy dz status =
(
	local sp
	local beam

	sp = [-((nx - 1) * dx) / 2, -((ny - 1) * dy) / 2, 0]

	-- vertical
	beam = draw_h_beam sp [0, 0, 90] (nz * dz) 305 300 15 15 18 18 106
	draw_beam_array beam nx ny 1 dx dy 0 status
	-- lateral
	beam = draw_h_beam [sp.x + 152.5, sp.y, sp.z + (dz - 300)] [90, 0, 90] (dx - 305) 600 200 11 17 22 22 106
	draw_beam_array beam (nx - 1) ny nz dx dy dz status
	-- longitudiral
	beam = draw_h_beam [sp.x, sp.y + 7.5, sp.z + (dz - 300)] [-90, 0, 0] (dy - 15) 600 200 11 17 22 22 106
	draw_beam_array beam nx (ny - 1) nz dx dy dz status
)

-- main program ---------------------------------------------------------------

rollout efx_rollout_struct_about "About"
(
	label l1 "Steel Structure"
	label l2 "Version 1.0"
	label l3 "Choi, Moon-Sun"
	label l4 "Copyright (c) 2001"
	label l5 "http://www.effectware.com"
	label l6 "aeroplus@effectware.com"
)

rollout efx_rollout_struct_param "Parameters"
(
	spinner nx "X counts:" type:#integer range:[1, 1000, 10]
	spinner ny "Y counts:" type:#integer range:[1, 1000, 6]
	spinner nz "Z counts:" type:#integer range:[1, 1000, 10]
	spinner dx "X spacing:" type:#float range:[1, 10000, 4000]
	spinner dy "Y spacing:" type:#float range:[1, 10000, 4000]
	spinner dz "Z spacing:" type:#float range:[1, 10000, 4000]
	label status
	button do_create " Create "

	on do_create pressed do
	(
		-- start creation
		undo on
		(
			draw_steel_structure nx.value ny.value nz.value dx.value dy.value dz.value status
			max tool zoomextents all
			status.text = ""
		)
	)
)

-- floater routines -----------------------------------------------------------

-- global variables

global efx_floater_struct
global efx_rollout_struct_about
global efx_rollout_struct_param

-- close old floater
if efx_floater_struct != undefined then closeRolloutFloater efx_floater_struct
-- add new floater
efx_floater_struct = newRolloutFloater "Steel Structure" 200 380
-- add rollout
addRollout efx_rollout_struct_about efx_floater_struct
addRollout efx_rollout_struct_param efx_floater_struct
